#include <iostream>
#include <string>
#include <iomanip>
#include "media.h"
#include "client.h"

using namespace std;

//========Media=======================================================
Media::Media(const string& format, const string& title)
	: format(format),
		title(title)
{
}

Media::Media(const Media& media)
	: format(media.format),
		title(media.title)
{
}

bool Media::isRented() const
{
	if (rents.size() > 0) return true;
	else return false;
}

void Media::removeRent()
{
	Rent* rent = rents[0];
	rents.erase(rents.begin());
	((Client*)rent->getClient())->removeRent(rent);
   delete rent;   
}

void Media::print(const string& space) const
{
	cout << space << format << ": " << title << endl;
}

//========Format=======================================================
Format::Format(const string& format,
					 		 const string& title,
					 		 int year, int length,
					 		 double price, double discount)
	: Media(format, title),
		year(year),
		length(length),
		price(price),
		discount(discount)
{
}

Format::Format(const Format& media)
	: Media(media),
		year(media.year),
		length(media.length),
		price(media.price),
		discount(media.discount)
{
}

/*Format::Format(const istream& is)
{
	if(string temp, !getline(is, title, '|').eof())
		{
			getline(is, temp, '|');
			year = atoi(temp.c_str());
			getline(is, temp, '|');
			length = atoi(temp.c_str());
		}
}*/


//========VHS=========================================================
VHS::VHS(int* pDisc,
				 const string& title, 
			   int year, int length,
			   const string& coding,
			   bool colour)
	: Format("VHS", title, year, length, pDisc[0], pDisc[3]),
		coding(coding),
		colour(colour)
{	
}

VHS::VHS(const VHS& format)
	: Format(format),
		coding(format.coding),
		colour(format.colour)
{	
}

VHS::VHS(int* pDisc, istream& is)
	: Format("VHS", "", 0, 0, pDisc[0], pDisc[3])
{
	string temp;
	if(!getline(is, title, '|').eof())
	{
		//getline(is, title, '|');
		getline(is, temp, '|');
		year = atoi(temp.c_str());
		getline(is, temp, '|');
		length = atoi(temp.c_str());
		getline(is, coding, '|');
		is >> temp;
		if (atoi(temp.c_str()) == 1) colour = true;
		else colour = false;
			
		getline(is, temp); //endl
	}
}
	
Media* VHS::clone() const
{
	return new VHS(*this);
}

void VHS::save(ostream& os) const
{
	os << format << "|" 
	   << title  << "|" 
	   << year   << "|" 
	   << length << "|" 
	   << coding << "|"
	   << colour << endl;
}

void VHS::print(const string& space) const
{
	cout << space << format << ": " 
	     << title  << ", "
	     << year   << " m. "
	     << length << " min, Koduote: "
	     << coding << ", Spavotas - ";
	     if (colour) cout << "TAIP";
	     				else cout << "NE";
	     cout << "." << endl;
}

double VHS::getPrice(const int day) const
{
	if(day > 3) return 3*price + (day - 3)*(price-discount);
	return day*price;
}
//========DVD=========================================================

DVD::DVD(int* pDisc,
				 const std::string& title,
				 int year, int length,
				 int region)
	: Format("DVD", title, year, length, pDisc[1], pDisc[4]),
		region(region)
{
}

DVD::DVD(const DVD& format)
	:	Format(format),
		region(format.region)
{
}

Media* DVD::clone() const
{
	return new DVD(*this);
}

DVD::DVD(int* pDisc, istream& is)
	: Format("DVD", "", 0, 0, pDisc[1], pDisc[4])
{
	string temp;
	if(!getline(is, title, '|').eof())
	{
		if(!getline(is, temp, '|').eof())
		{
			year = atoi(temp.c_str());
			if(!getline(is, temp, '|').eof())
			{
				length = atoi(temp.c_str());
				is >> temp;
				if(!is.eof())
					{
						region = atoi(temp.c_str());
						getline(is, temp);
					}
			}
		}	
	}
}

void DVD::save(ostream& os) const
{
	os << format << "|" 
	   << title  << "|" 
	   << year   << "|" 
	   << length << "|" 
	   << region << endl;
}

void DVD::print(const string& space) const
{
	cout << space << format << ": " 
	     << title  << ", "
	     << year   << " m. "
	     << length << " min, Regionas: "
	     << region << endl;
}

double DVD::getPrice(const int day) const
{
	if(day > 5) return 5*price + (day - 5)*(price-discount);
	return day*price;
}

//========VCD=========================================================
VCD::VCD(int* pDisc,
				 const std::string& title,
				 int year, int length,
				 bool dSided)
	: Format("VCD", title, year, length, pDisc[2], pDisc[5]),
		doubleSided(dSided)
{
}

VCD::VCD(const VCD& format)
	:	Format(format),
		doubleSided(format.doubleSided)
{
}

Media* VCD::clone() const
{
	return new VCD(*this);
}

VCD::VCD(int* pDisc, istream& is)
	: Format("VCD", "", 0, 0, pDisc[2], pDisc[5])
{
	string temp;
	if(!getline(is, title, '|').eof())
	{
		if(!getline(is, temp, '|').eof())
		{
			year = atoi(temp.c_str());
			if(!getline(is, temp, '|').eof())
			{
				length = atoi(temp.c_str());
				is >> temp;
				if(!is.eof())
					{
						//dSided = atoi(temp.c_str());
						if(temp == "1") doubleSided = true;
						else if (temp =="0") doubleSided =false;
						getline(is, temp);
					}
			}
		}	
	}
}

void VCD::save(ostream& os) const
{
	os << format << "|" 
	   << title  << "|" 
	   << year   << "|" 
	   << length << "|" 
	   << doubleSided << endl;
}

void VCD::print(const string& space) const
{
	cout << space << format << ": " 
	     << title  << ", "
	     << year   << " m. "
	     << length << " min, ";
	if(doubleSided) cout << "Dvipusis";
	else cout << "Vienpusis";
	cout << endl;
}

double VCD::getPrice(const int day) const
{
	return day*price-discount;
}

//========SET=========================================================
SET::SET(const string& title)
	: Media("SET", title)
{
}

SET::SET(const SET& media)
	: Media(media)
{
	for(int i=0;i<media.getMediaSize();i++)
		addMediaCopy(media.getMedia(i));
}

Media* SET::clone() const
{
	return new SET(*this);
}

void SET::addMediaCopy(const Media* nMedia)
{
		media.push_back(nMedia->clone());
}

double SET::getPrice(int days) const
{
	double price = 0;
	for(int i=0;i<media.size();i++)
		price+= media[i]->getPrice(days);
	return price;
}

void SET::save(ostream& os) const
{
	os << format << "|"
		 << title << "|"
		 << media.size() << "|";
	//int mSize = media.size();
	//mSize--;
	int i;
	for( i=0; i < media.size(); i++)
		os << media[i]->getTitle() << "|";
	os << endl;
	//os << media[i]->getTitle() << endl;
}

void SET::print(const string& space) const
{
	Media::print(space);
	for(int i = 0;i<media.size();i++)
		media[i]->print(space+"     ");
}

bool SET::checkDuplicate(const string& title) const
{
	bool dup = false;
	for(int i=0; i < getMediaSize();i++)
	//{
		//out << "#SYSTEM: " << title << " <> " << media[i]->getTitle() << endl;
		if(title == media[i]->getTitle()) 
			{
				dup = true;
				break;
			}
	//}
	return dup;
}
